# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['xoinvader', 'xoinvader.application', 'xoinvader.tests']

package_data = \
{'': ['*'],
 'xoinvader': ['config/charges.json',
               'config/charges.json',
               'config/charges.json',
               'config/charges.json',
               'config/path.json',
               'config/path.json',
               'config/path.json',
               'config/path.json',
               'config/ships.json',
               'config/ships.json',
               'config/ships.json',
               'config/ships.json',
               'config/weapons.json',
               'config/weapons.json',
               'config/weapons.json',
               'config/weapons.json',
               'res/*',
               'res/gfx/*',
               'res/snd/*'],
 'xoinvader.tests': ['fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_duplicate_names.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_no_name.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/chunk_normal.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/no_chunks.bg',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_corrupted',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_empty',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid',
                     'fixtures/scoreboard_valid']}

install_requires = \
['pygame', 'tornado>=4,<5']

setup_kwargs = {
    'name': 'xoinvader',
    'version': '0.1.1',
    'description': '(not so) small python curses space game',
    'long_description': None,
    'author': 'Pavel Kulyov',
    'author_email': 'kulyov.pavel@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3,<4',
}


setup(**setup_kwargs)
